﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2010/07/27 初版
//
// FVC05 1ch only Sample Program
// -----------------------------------------------------------------
namespace FVC05_1chSample
{
    public partial class Form_FVC051CHSAMPLE : Form
    {

        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //FVC05クラス
        private FVIL.Video.CFviVideoFVC05 m_video = new FVIL.Video.CFviVideoFVC05();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        //フリーランの状態
        private bool m_flgFreerun;
        
        public Form_FVC051CHSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            dialogLicence licence_dialog = new dialogLicence();
            if ((licence_dialog.ShowDialog(this) != DialogResult.OK))
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            cFviImageView1.Image = m_image;

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。

            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //チャネルコンボボックス初期化
            ComboBoxChannel.Items.Clear();
            ComboBoxChannel.Items.Add("0CH");
            ComboBoxChannel.Items.Add("1CH");
            ComboBoxChannel.SelectedIndex = 0;

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //２値化コンボボックス初期化
            ComboBoxBinarize.Items.Clear();
            ComboBoxBinarize.Items.Add("グレイ");
            ComboBoxBinarize.Items.Add("２値");
            ComboBoxBinarize.SelectedIndex = 0;

        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();
            //iniファイル設定画面

            //iniファイルの選択
            m_strIniFilePath = "";
            if ((formbroseini.ShowDialog(this) == DialogResult.OK))
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if ((m_strIniFilePath.Length > 0))
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video.IsOpen())
                    {
                        m_video.Close();
                        m_flgVideoOpened = false;
                    }

                    //デバイスオープン
                    m_video.Open(-1, 0);
                    m_flgVideoOpened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //iniファイルをコンポーネントに設定
                    if ((m_strIniFilePath.Length > 0))
                    {
                        m_video.LoadIniFile(m_strIniFilePath);
                    }

                    //取込画像を正規化
                    m_video.ChangeImageSize(m_image);

                    //ステータス表示の更新
                    //入力チャネルの取得
                    ComboBoxChannel.SelectedIndex = (int)m_video.InputChannel;
                    //トリガモードの取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    //カメラ露光時間の取得
                    NumericUpDownExposure.Value = m_video.ExposureTime;
                    //選択されているチャネルのステータスを取得
                    if ((m_video.InputChannel == 0))
                    {
                        //ビデオ入力オフセット(CH0)の取得
                        NumericUpDownOffset.Value = m_video.VideoOffsetCH0;
                        //２値化設定(CH0)の取得
                        ComboBoxBinarize.SelectedIndex = (int)m_video.BinarizeCH0.Mode;
                        //２値化閾値(CH0)の取得
                        NumericUpDownThreshold.Value = m_video.BinarizeCH0.Threshold;
                    }
                    else
                    {
                        //ビデオ入力オフセット(CH1)の取得
                        NumericUpDownOffset.Value = m_video.VideoOffsetCH1;
                        //２値化設定(CH1)の取得
                        ComboBoxBinarize.SelectedIndex = (int)m_video.BinarizeCH1.Mode;
                        //２値化閾値(CH1)の取得
                        NumericUpDownThreshold.Value = m_video.BinarizeCH1.Threshold;
                    }

                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;

            try
            {
                m_timer.Start();
                //時間測定開始
                //カメラ取り込み
                m_video.GrabImageSync(m_image);
                msec = m_timer.Stop();
                //時間測定終了
                TextBoxTime.Text = msec.ToString("0.000");
                //測定時間表示
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            if ((m_flgFreerun == true))
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
            }
            else
            {
                ButtonGrabContinue.Text = "Stop";
                ButtonGrabOne.Enabled = false;
                m_flgFreerun = true;
                timer1.Interval = 100;
                timer1.Start();
            }
            Application.DoEvents();

        }

        /// <summary>
        /// フリーラン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            double msec = 0;

            timer1.Stop();

            try
            {
                if ((m_flgVideoOpened == true))
                {
                    //時間測定開始
                    m_timer.Start();
                    //カメラ取り込み
                    m_video.GrabImageSync(m_image);
                    //時間測定終了
                    msec = m_timer.Stop();
                    //測定時間表示
                    TextBoxTime.Text = msec.ToString("0.000");
                }
                timer1.Enabled = true;

            }
            catch (FVIL.CFviException ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// チャネル設定コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// チャネルの選択。
        /// </remarks>
        private void ComboBoxChannel_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    m_video.InputChannel = (uint)ComboBoxChannel.SelectedIndex;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Offset値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセットの設定。
        /// </remarks>
        private void NumericUpDownOffset_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    //選択されているチャネル側に設定
                    if ((ComboBoxChannel.SelectedIndex == 0))
                    {
                        m_video.VideoOffsetCH0 = (int)NumericUpDownOffset.Value;
                    }
                    else
                    {
                        m_video.VideoOffsetCH1 = (int)NumericUpDownOffset.Value;
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Binarize Mode設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化取込設定。
        /// </remarks>
        private void ComboBoxBinarize_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video.Binarize;
                    bin.Mode = (uint)ComboBoxBinarize.SelectedIndex;
                    //選択されているチャネル側に設定
                    if ((ComboBoxChannel.SelectedIndex == 0))
                    {
                        m_video.BinarizeCH0 = bin;
                    }
                    else
                    {
                        m_video.BinarizeCH1 = bin;
                    }
                    //取込画像を正規化
                    m_video.ChangeImageSize(m_image);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Bin Threshold値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化しきい値の設定。
        /// </remarks>
        private void NumericUpDownThreshold_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if ((m_flgVideoOpened == true))
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video.Binarize;
                    bin.Threshold = (uint)NumericUpDownThreshold.Value;
                    //選択されているチャネル側に設定
                    if ((ComboBoxChannel.SelectedIndex == 0))
                    {
                        m_video.BinarizeCH0 = bin;
                    }
                    else
                    {
                        m_video.BinarizeCH1 = bin;
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります

            //表示倍率の設定。
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if ((Value <= 0))
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Enable = true;
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if ((m_flgFreerun == true)) {
		        ButtonGrabContinue.Text = "Grab Continue";
		        ButtonGrabOne.Enabled = true;
		        m_flgFreerun = false;
		        timer1.Enabled = false;
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if ((ofdlg.ShowDialog(this) == DialogResult.OK))
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC051CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }
    }
}